@@ -100,6 +100,14 @@ class GuidedContext:
 
     # pylint: disable=too-many-statements
     def guided_prompts(self, parameter_override_keys):
+        """
+        Start an interactive cli prompt to collection information for deployment
+
+        Parameters
+        ----------
+        parameter_override_keys
+            The keys of parameters to override, for each key, customers will be asked to provide a value
+        """
         default_stack_name = self.stack_name or "sam-app"
         default_region = self.region or get_session().get_config_variable("region") or "us-east-1"
         default_capabilities = self.capabilities[0] or ("CAPABILITY_IAM",)
@@ -122,7 +130,7 @@ class GuidedContext:
         input_parameter_overrides = self.prompt_parameters(
             parameter_override_keys, self.parameter_overrides_from_cmdline, self.start_bold, self.end_bold
         )
-        stacks = SamLocalStackProvider.get_stacks(
+        stacks, _ = SamLocalStackProvider.get_stacks(
             self.template_file, parameter_overrides=sanitize_parameter_overrides(input_parameter_overrides)
         )
         image_repositories = self.prompt_image_repository(stacks)
@@ -161,13 +169,9 @@ class GuidedContext:
                 type=click.STRING,
             )
 
-        s3_bucket = self.s3_bucket
-        if s3_bucket:
-            click.echo(f"\n\t\tUsing defined S3 bucket: {s3_bucket}")
-        else:
-            s3_bucket = manage_stack(profile=self.profile, region=region)
-            click.echo(f"\n\t\tManaged S3 bucket: {s3_bucket}")
-            click.echo("\t\tA different default S3 bucket can be set in samconfig.toml")
+        s3_bucket = manage_stack(profile=self.profile, region=region)
+        click.echo(f"\n\t\tManaged S3 bucket: {s3_bucket}")
+        click.echo("\t\tA different default S3 bucket can be set in samconfig.toml")
 
         self.guided_stack_name = stack_name
         self.guided_s3_bucket = s3_bucket
@@ -197,6 +201,15 @@ class GuidedContext:
                     raise GuidedDeployFailedError(msg="Security Constraints Not Satisfied!")
 
     def prompt_code_signing_settings(self, stacks: List[Stack]):
+        """
+        Prompt code signing settings to ask whether customers want to code sign their code and
+        display signing details.
+
+        Parameters
+        ----------
+        stacks : List[Stack]
+            List of stacks to search functions and layers
+        """
         (functions_with_code_sign, layers_with_code_sign) = signer_config_per_function(stacks)
 
         # if no function or layer definition found with code signing, skip it
@@ -273,6 +286,20 @@ class GuidedContext:
         return _prompted_param_overrides
 
     def prompt_image_repository(self, stacks: List[Stack]):
+        """
+        Prompt for the image repository to push the images.
+        For each image function found in build artifacts, it will prompt for an image repository.
+
+        Parameters
+        ----------
+        stacks : List[Stack]
+            List of stacks to look for image functions.
+
+        Returns
+        -------
+        Dict
+            A dictionary contains image function logical ID as key, image repository as value.
+        """
         image_repositories = {}
         artifacts_format = get_template_artifacts_format(template_file=self.template_file)
         if IMAGE in artifacts_format:
