@@ -53,3 +53,9 @@ class LambdaImagesTemplateException(UserException):
     """
     Exception class when multiple Lambda Image app templates are found for any runtime
     """
+
+
+class ContainersInitializationException(UserException):
+    """
+    Exception class when SAM is not able to initialize any of the lambda functions containers
+    """
