@@ -9,7 +9,7 @@ from json import JSONDecodeError
 import click
 
 from samcli.cli.cli_config_file import configuration_option, TomlProvider
-from samcli.cli.main import pass_context, common_options
+from samcli.cli.main import pass_context, common_options, print_cmdline_args
 from samcli.lib.utils.version_checker import check_newer_version
 from samcli.local.common.runtime_template import RUNTIMES, SUPPORTED_DEP_MANAGERS, LAMBDA_IMAGES_RUNTIMES
 from samcli.lib.telemetry.metric import track_command
@@ -204,6 +204,7 @@ def non_interactive_validation(func):
 @pass_context
 @track_command
 @check_newer_version
+@print_cmdline_args
 def cli(
     ctx,
     no_interactive,
@@ -220,6 +221,9 @@ def cli(
     config_file,
     config_env,
 ):
+    """
+    `sam init` command entry point
+    """
     do_cli(
         ctx,
         no_interactive,
@@ -254,6 +258,9 @@ def do_cli(
     extra_context,
     auto_clone=True,
 ):
+    """
+    Implementation of the ``cli`` method
+    """
 
     from samcli.commands.init.init_generator import do_generate
     from samcli.commands.init.interactive_init_flow import do_interactive
