@@ -6,7 +6,7 @@ from functools import partial
 import click
 
 from samcli.cli.cli_config_file import configuration_option, TomlProvider
-from samcli.cli.main import pass_context, common_options, aws_creds_options
+from samcli.cli.main import pass_context, common_options, aws_creds_options, print_cmdline_args
 from samcli.cli.types import ImageRepositoryType, ImageRepositoriesType
 from samcli.commands.package.exceptions import PackageResolveS3AndS3SetError, PackageResolveS3AndS3NotSetError
 from samcli.lib.cli_validation.image_repository_validation import image_repository_validation
@@ -134,6 +134,7 @@ The following resources and their property locations are supported.
 @track_command
 @check_newer_version
 @track_template_warnings([CodeDeployWarning.__name__, CodeDeployConditionWarning.__name__])
+@print_cmdline_args
 def cli(
     ctx,
     template_file,
@@ -152,7 +153,9 @@ def cli(
     config_file,
     config_env,
 ):
-
+    """
+    `sam package` command entry point
+    """
     # All logic must be implemented in the ``do_cli`` method. This helps with easy unit testing
 
     do_cli(
@@ -191,6 +194,10 @@ def do_cli(
     profile,
     resolve_s3,
 ):
+    """
+    Implementation of the ``cli`` method
+    """
+
     from samcli.commands.package.package_context import PackageContext
 
     if resolve_s3:
