@@ -38,6 +38,8 @@ def generate_project(
     location: Path, optional
         Git, HTTP, Local path or Zip containing cookiecutter template
         (the default is None, which means no custom template)
+    package_type : Optional[str]
+        Optional string representing the package type, 'Zip' or 'Image', see samcli/lib/utils/packagetype.py
     runtime: str
         Lambda Runtime
     dependency_manager: str, optional
@@ -50,6 +52,8 @@ def generate_project(
     no_input : bool, optional
         Whether to prompt for input or to accept default values
         (the default is False, which prompts the user for values it doesn't know for baking)
+    extra_context : Optional[Dict]
+        An optional dictionary, the extra cookiecutter context
 
     Raises
     ------
