@@ -2,8 +2,7 @@
 Class that provides functions from a given SAM template
 """
 import logging
-import posixpath
-from typing import Dict, List, Optional, cast, Iterator
+from typing import Dict, List, Optional, cast, Iterator, Any
 
 from samcli.commands.local.cli_common.user_exceptions import InvalidLayerVersionArn
 from samcli.lib.providers.exceptions import InvalidLayerReference
@@ -11,6 +10,7 @@ from samcli.lib.utils.colors import Colored
 from samcli.lib.utils.packagetype import ZIP, IMAGE
 from .provider import Function, LayerVersion, Stack
 from .sam_base_provider import SamBaseProvider
+from .sam_stack_provider import SamLocalStackProvider
 
 LOG = logging.getLogger(__name__)
 
@@ -23,7 +23,7 @@ class SamFunctionProvider(SamBaseProvider):
     It may or may not contain a function.
     """
 
-    def __init__(self, stacks: List[Stack], ignore_code_extraction_warnings=False):
+    def __init__(self, stacks: List[Stack], ignore_code_extraction_warnings: bool = False) -> None:
         """
         Initialize the class with SAM template data. The SAM template passed to this provider is assumed
         to be valid, normalized and a dictionary. It should be normalized by running all pre-processing
@@ -43,8 +43,8 @@ class SamFunctionProvider(SamBaseProvider):
         for stack in stacks:
             LOG.debug("%d resources found in the stack %s", len(stack.resources), stack.stack_path)
 
-        # Store a map of function name to function information for quick reference
-        self.functions = self._extract_functions(self.stacks, ignore_code_extraction_warnings)
+        # Store a map of function full_path to function information for quick reference
+        self.functions = SamFunctionProvider._extract_functions(self.stacks, ignore_code_extraction_warnings)
 
         self._deprecated_runtimes = {"nodejs4.3", "nodejs6.10", "nodejs8.10", "dotnetcore2.0"}
         self._colored = Colored()
@@ -53,8 +53,9 @@ class SamFunctionProvider(SamBaseProvider):
         """
         Returns the function given name or LogicalId of the function. Every SAM resource has a logicalId, but it may
         also have a function name. This method searches only for LogicalID and returns the function that matches.
-        If it is in a nested stack, "name" can be prefixed with stack path to avoid ambiguity
-        it.
+        If it is in a nested stack, "name" can be prefixed with stack path to avoid ambiguity.
+        For example, if a function with name "FunctionA" is located in StackN, which is a nested stack in root stack,
+          either "StackN/FunctionA" or "FunctionA" can be used.
 
         :param string name: Name of the function
         :return Function: namedtuple containing the Function information if function is found.
@@ -65,12 +66,12 @@ class SamFunctionProvider(SamBaseProvider):
         if not name:
             raise ValueError("Function name is required")
 
-        for f in self.get_all():
-            if posixpath.join(f.stack_path, f.name) == name or f.name == name:
-                self._deprecate_notification(f.runtime)
-                return f
+        # support lookup by full_path
+        if name in self.functions:
+            return self.functions.get(name)
 
-            if posixpath.join(f.stack_path, f.functionname) == name or f.functionname == name:
+        for f in self.get_all():
+            if name in (f.name, f.functionname):
                 self._deprecate_notification(f.runtime)
                 return f
 
@@ -97,18 +98,18 @@ class SamFunctionProvider(SamBaseProvider):
             yield function
 
     @staticmethod
-    def _extract_functions(stacks: List[Stack], ignore_code_extraction_warnings=False) -> Dict[str, Function]:
+    def _extract_functions(stacks: List[Stack], ignore_code_extraction_warnings: bool = False) -> Dict[str, Function]:
         """
         Extracts and returns function information from the given dictionary of SAM/CloudFormation resources. This
         method supports functions defined with AWS::Serverless::Function and AWS::Lambda::Function
 
-        :param dict resources_by_stack: Dictionary of SAM/CloudFormation resources by stack
+        :param stacks: List of SAM/CloudFormation stacks to extract functions from
         :param bool ignore_code_extraction_warnings: suppress log statements on code extraction from resources.
-        :return dict(string : samcli.commands.local.lib.provider.Function): Dictionary of function LogicalId to the
+        :return dict(string : samcli.commands.local.lib.provider.Function): Dictionary of function full_path to the
             Function configuration object
         """
 
-        result = {}
+        result: Dict[str, Function] = {}  # a dict with full_path as key and extracted function as value
         for stack in stacks:
             for name, resource in stack.resources.items():
 
@@ -121,29 +122,29 @@ class SamFunctionProvider(SamBaseProvider):
 
                 if resource_type == SamFunctionProvider.SERVERLESS_FUNCTION:
                     layers = SamFunctionProvider._parse_layer_info(
-                        stack.stack_path,
+                        stack,
                         resource_properties.get("Layers", []),
-                        stack.resources,
                         ignore_code_extraction_warnings=ignore_code_extraction_warnings,
                     )
-                    result[name] = SamFunctionProvider._convert_sam_function_resource(
-                        stack.stack_path,
+                    function = SamFunctionProvider._convert_sam_function_resource(
+                        stack,
                         name,
                         resource_properties,
                         layers,
                         ignore_code_extraction_warnings=ignore_code_extraction_warnings,
                     )
+                    result[function.full_path] = function
 
                 elif resource_type == SamFunctionProvider.LAMBDA_FUNCTION:
                     layers = SamFunctionProvider._parse_layer_info(
-                        stack.stack_path,
+                        stack,
                         resource_properties.get("Layers", []),
-                        stack.resources,
                         ignore_code_extraction_warnings=ignore_code_extraction_warnings,
                     )
-                    result[name] = SamFunctionProvider._convert_lambda_function_resource(
-                        stack.stack_path, name, resource_properties, layers
+                    function = SamFunctionProvider._convert_lambda_function_resource(
+                        stack, name, resource_properties, layers
                     )
+                    result[function.full_path] = function
 
                 # We don't care about other resource types. Just ignore them
 
@@ -151,7 +152,7 @@ class SamFunctionProvider(SamBaseProvider):
 
     @staticmethod
     def _convert_sam_function_resource(
-        stack_path: str,
+        stack: Stack,
         name: str,
         resource_properties: Dict,
         layers: List[LayerVersion],
@@ -195,12 +196,18 @@ class SamFunctionProvider(SamBaseProvider):
             LOG.debug("Found Serverless function with name='%s' and ImageUri='%s'", name, imageuri)
 
         return SamFunctionProvider._build_function_configuration(
-            stack_path, name, codeuri, resource_properties, layers, inlinecode, imageuri
+            stack,
+            name,
+            codeuri,
+            resource_properties,
+            layers,
+            inlinecode,
+            imageuri,
         )
 
     @staticmethod
     def _convert_lambda_function_resource(
-        stack_path: str, name: str, resource_properties: Dict, layers: List[LayerVersion]
+        stack: Stack, name: str, resource_properties: Dict, layers: List[LayerVersion]
     ) -> Function:
         """
         Converts a AWS::Lambda::Function resource to a Function configuration usable by the provider.
@@ -243,12 +250,18 @@ class SamFunctionProvider(SamBaseProvider):
             LOG.debug("Found Lambda function with name='%s' and Imageuri='%s'", name, imageuri)
 
         return SamFunctionProvider._build_function_configuration(
-            stack_path, name, codeuri, resource_properties, layers, inlinecode, imageuri
+            stack,
+            name,
+            codeuri,
+            resource_properties,
+            layers,
+            inlinecode,
+            imageuri,
         )
 
     @staticmethod
     def _build_function_configuration(
-        stack_path: str,
+        stack: Stack,
         name: str,
         codeuri: Optional[str],
         resource_properties: Dict,
@@ -275,8 +288,14 @@ class SamFunctionProvider(SamBaseProvider):
         samcli.commands.local.lib.provider.Function
             Function configuration
         """
+        metadata = resource_properties.get("Metadata", None)
+        if metadata and "DockerContext" in metadata:
+            metadata["DockerContext"] = SamLocalStackProvider.normalize_resource_path(
+                stack.location, metadata["DockerContext"]
+            )
+
         return Function(
-            stack_path=stack_path,
+            stack_path=stack.stack_path,
             name=name,
             functionname=resource_properties.get("FunctionName", name),
             packagetype=resource_properties.get("PackageType", ZIP),
@@ -284,23 +303,22 @@ class SamFunctionProvider(SamBaseProvider):
             memory=resource_properties.get("MemorySize"),
             timeout=resource_properties.get("Timeout"),
             handler=resource_properties.get("Handler"),
-            codeuri=codeuri,
+            codeuri=SamLocalStackProvider.normalize_resource_path(stack.location, codeuri) if codeuri else None,
             imageuri=imageuri if imageuri else resource_properties.get("ImageUri"),
             imageconfig=resource_properties.get("ImageConfig"),
             environment=resource_properties.get("Environment"),
             rolearn=resource_properties.get("Role"),
             events=resource_properties.get("Events"),
             layers=layers,
-            metadata=resource_properties.get("Metadata", None),
+            metadata=metadata,
             inlinecode=inlinecode,
             codesign_config_arn=resource_properties.get("CodeSigningConfigArn", None),
         )
 
     @staticmethod
     def _parse_layer_info(
-        stack_path: str,
-        list_of_layers: List[LayerVersion],
-        resources: Dict,
+        stack: Stack,
+        list_of_layers: List[Any],
         ignore_code_extraction_warnings: bool = False,
     ) -> List[LayerVersion]:
         """
@@ -308,10 +326,13 @@ class SamFunctionProvider(SamBaseProvider):
 
         Parameters
         ----------
-        list_of_layers List(str)
-            List of layers that are defined within the Layers Property on a function
-        resources dict
-            The Resources dictionary defined in a template
+        stack : Stack
+            The stack the layer is defined in
+        list_of_layers : List[Any]
+            List of layers that are defined within the Layers Property on a function,
+            layer can be defined as string or Dict, in case customers define it in other types, use "Any" here.
+        ignore_code_extraction_warnings : bool
+            Whether to print warning when codeuri is not a local pth
 
         Returns
         -------
@@ -340,7 +361,7 @@ class SamFunctionProvider(SamBaseProvider):
                     LayerVersion(
                         layer,
                         None,
-                        stack_path=stack_path,
+                        stack_path=stack.stack_path,
                     )
                 )
                 continue
@@ -349,7 +370,7 @@ class SamFunctionProvider(SamBaseProvider):
             # When running locally, we need to follow that Ref so we can extract the local path to the layer code.
             if isinstance(layer, dict) and layer.get("Ref"):
                 layer_logical_id = cast(str, layer.get("Ref"))
-                layer_resource = resources.get(layer_logical_id)
+                layer_resource = stack.resources.get(layer_logical_id)
                 if not layer_resource or layer_resource.get("Type", "") not in (
                     SamFunctionProvider.SERVERLESS_LAYER,
                     SamFunctionProvider.LAMBDA_LAYER,
@@ -359,7 +380,7 @@ class SamFunctionProvider(SamBaseProvider):
                 layer_properties = layer_resource.get("Properties", {})
                 resource_type = layer_resource.get("Type")
                 compatible_runtimes = layer_properties.get("CompatibleRuntimes")
-                codeuri = None
+                codeuri: Optional[str] = None
 
                 if resource_type == SamFunctionProvider.LAMBDA_LAYER:
                     codeuri = SamFunctionProvider._extract_lambda_function_code(layer_properties, "Content")
@@ -372,10 +393,10 @@ class SamFunctionProvider(SamBaseProvider):
                 layers.append(
                     LayerVersion(
                         layer_logical_id,
-                        codeuri,
+                        SamLocalStackProvider.normalize_resource_path(stack.location, codeuri) if codeuri else None,
                         compatible_runtimes,
                         layer_resource.get("Metadata", None),
-                        stack_path=stack_path,
+                        stack_path=stack.stack_path,
                     )
                 )
 
